<?php
/*
	Plugin Name: WordTwit
	Plugin URI: http://www.bravenewcode.com/wordtwit
	Description: Posts updates to Twitter about entries automatically. Includes many configuarble options, statistics and more.
	Author: Dale Mugford & Duane Storey (BraveNewCode)
	Author URI: http://www.bravenewcode.com
	Version: 2.2.4

	# Some ideas taken from http://twitter.slawcup.com/twitter.class.phps
	
	# This plugin is free software; you can redistribute the code aspects of it and/or
	# modify the code under the terms of the GNU Lesser General Public
	# License as published by the Free Software Foundation; either
	# version 2.1 of the License, or (at your option) any later version.
	
	# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
	# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
	# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
	# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
	# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
	# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
	# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
	#
	# See the GNU lesser General Public License for more details.
*/

require_once( 'compat.php' );

define( 'WORDTWIT_CONSUMER_KEY', 'J3hOpNubCjyJCLOSc6f3A' );
define( 'WORDTWIT_CONSUMER_SECRET', 'lfD2BVLuIUWkARnDKXU0ffT3a5JpJXtUW0JWVVLVbw' );
define( 'WORDTWIT_CACHE_TIME', 600 );

require_once( ABSPATH . 'wp-includes/class-snoopy.php' );

require_once( 'xml.php' );
require_once( 'tinyurl.php' );
require_once( 'include/widgets.php' );

$twit_plugin_name = 'WordTwit';
$twit_plugin_prefix = 'wordtwit_';
$wordtwit_version = '2.2.4';
$wordtwit_cache_time = 500;

// set up hooks for WordPress
add_action( 'publish_post', 'post_now_published' );
add_action( 'admin_head', 'wordtwit_admin_css' );
add_action( 'wp_head', 'wordtwit_head' );
add_action( 'save_post', 'wordtwit_save_post' );
add_filter( 'init', 'wordtwit_init');
add_action( 'delete_post', 'wordtwit_delete_post' );
add_action( 'query_vars', 'wordtwit_query_vars' );
add_action( 'parse_request', 'wordtwit_parse_request' );

function wordtwit_query_vars( $vars ) {
	$vars[] = "wordtwit";
	return $vars;
}

function wordtwit_parse_request( $wp ) {
	if (array_key_exists( 'wordtwit', $wp->query_vars ) ) {
		switch( $wp->query_vars['wordtwit'] ) {
			case 'create':
				include( 'ajax/create-url.php' );
				break;
			case 'news':
				include( 'ajax/news.php' );
				break;
			case 'tweets':
				include( 'ajax/tweets.php' );
				break;
		}			exit;
	}	
}

function wordtwit_avatar( $avatar ) {
	global $comment;
	
	if ( $comment->comment_type == 'wordtwit' ) {
		echo "<img src=\"" . $comment->comment_agent . "\" alt=\"\" />";
	} else return $avatar;
}

function wordtwit_head() {
	echo '<link rel="stylesheet" type="text/css" media="screen" href="' . compat_get_plugin_url('wordtwit') . '/css/style.css" />';	
}

//Add a link to settings on the plugin listings page
function wordtwit_settings_link( $links, $file ) {
 	if( $file == 'wordtwit/wordtwit.php' && function_exists( "admin_url" ) ) {
		$settings_link = '<a href="' . admin_url( 'options-general.php?page=wordtwit.php' ) . '">' . __('Settings') . '</a>';
		array_unshift( $links, $settings_link ); // before other links
	}
	return $links;
}

$wordtwit_defaults = array(
	'username' => '',
	'password' => '',
	'alternate_domain' => '',
	'tags' => array(),
	'reverse' => false,
	'activation_time' => 0,
	'message' => 'New blog posting, [title] - [link]',
	'url_type' => 'miud.in',
	'enable_banner' => true,
	'enable_content_conversion' => false,
	'oauth_token' => false,
	'enable-utm' => false
);

function wordtwit_get_settings() {
	global $wordtwit_defaults;
	
	$settings = $wordtwit_defaults;
	
	$wordpress_settings = get_option( 'wordtwit_settings' );
	if ( $wordpress_settings ) {
		foreach( $wordpress_settings as $key => $value ) {
			$settings[ $key ] = $value;
		}	
	}
	
	return $settings;
}

function wordtwit_save_post( $post_id ) {
	if ( !wp_verify_nonce( $_POST['wordtwit_nonce'], 'WordTwit' ) ) {
		return $post_id;
	}
	
	delete_post_meta( $post_id, 'wordtwit_hashtags' );
}

function wordtwit_delete_post( $id ) {
	global $wpdb;
	global $table_prefix;
	
	$sql = $wpdb->prepare( 'DELETE FROM ' . $table_prefix . 'tweet_urls WHERE post_id = %d', $id );
	$wpdb->query( $sql );
}

function wordtwit_post_box() {
	echo '<input type="checkbox" name="wordtwit-twitter-post" id="wordtwit-twitter-post" />';		
	echo '<label for="wordtwit-twitter-post">' . __( "Twitter This Post", "wordtwit" ) . '</label><br /><br />';
	echo '<label for="wordtwit-hash-tags">' . __( "Hash Tags", "wordtwit" ) . '</label>';
	echo '<input type="text" name="wordtwit-hash-tags" id="wordtwit-hash-tags" />';
	echo '<input type="hidden" name="wordtwit_nonce" id="wordtwit_nonce" value="' . wp_create_nonce( 'WordTwit' ) . '" />';
}

function wordtwit_admin_menu() {
	if( function_exists( 'add_meta_box' ) ) {
		add_meta_box( 'wordtwit-box', __( 'WordTwit Post Management', 'wordtwit' ), 'wordtwit_post_box', 'post', 'side' );
	}
	
}

function wordtwit_is_valid() {
	$settings = wordtwit_get_settings();
	
	return ( isset( $settings['username'] ) && isset( $settings['password'] ) );	
}

function wordtwit_friendly_date( $datetime ) {
	$now = time();
	$datetime = $now - strftime( "%s", $datetime );
	
	$minutes = floor( $datetime / (60) );
	$hours = $datetime / (60*60);
	$days = $datetime / (60*60*24);
	
	if ( $minutes == 0 ) {
		return __( "Just now", "wordtwit" );	
	} else if ( $minutes < 120 && $hours < 2 ) {
		return sprintf( __( "About %d minutes ago", "wordtwit" ), $minutes );
	} else if ( $hours < 48 ) {
		return sprintf( __( "About %d hours ago", "wordtwit" ), $hours );
	} else {
		return sprintf( __( "About %d days ago", "wordtwit" ), $days );
	}
}

function wordtwit_init() {
	global $twit_plugin_prefix;
	global $wordtwit_cache_time;
	global $wpdb;
	global $table_prefix;
	
	$settings = wordtwit_get_settings();
	
	if ( $settings['enable_content_conversion'] ) {
		add_action( 'the_content', 'wordtwit_content' );	
	}
	
	if ( $settings['activation_time'] == 0 ) {
		$settings['activation_time'] = time();
		
		wordtwit_save_settings( $settings );	
	}
	
	if ( strpos( $_SERVER["REQUEST_URI"], "wp-admin" ) !== false ) {
		if ( isset( $_GET['page'] ) && $_GET['page'] == 'wordtwit.php' ) {
		wp_enqueue_script( 'fancybox', compat_get_plugin_url('wordtwit') . '/js/fancybox_1.2.5.js', array( 'jquery' ) );
		wp_enqueue_script( 'wordtwitadmin', compat_get_plugin_url('wordtwit') . '/js/wordtwit_admin_2.1.js', array( 'jquery' ) );
	}
}
	
	wordtwit_check_table();
	
	// check TinyURl
	$url = $_SERVER["REQUEST_URI"];
	
	if ( strlen( $url ) < 6 ) {
		// might be a tiny URL
		$params = explode( '/', $url );
		if ( count( $params ) == 2 ) {
			$tiny_url = $params[1];
			
			$sql = $wpdb->prepare( "SELECT original FROM " . $table_prefix . "tweet_urls WHERE url = %s",  $tiny_url );
			$result = $wpdb->get_row( $sql );
			if ( $result ) {
				$sql = $wpdb->prepare( "UPDATE " . $table_prefix . "tweet_urls SET views = views + 1 WHERE url = %s",  $tiny_url );
				$wpdb->query( $sql );
				
				$this_site = strtolower( wordtwit_short_home( get_bloginfo('home') ) );
				$short_url = strtolower( str_replace( 'www.', '', $result->original ) );
				if ( strpos( $short_url, $this_site ) === false && $settings['enable_banner'] ) {
					// external site
					header( "Content-type: text/html" );
					
					$profile = $settings['profile'];
					
					
					echo "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">
<html xmlns=\"http://www.w3.org/1999/xhtml\"  dir=\"ltr\" lang=\"en-US\">\n";
					echo "\t<head>\n";
					echo "\t<title>" . $result->original . " - Courtesy of " . $profile['user']['name'] . "</title>\n";
					echo "\t<link rel=\"stylesheet\" type=\"text/css\" href=\"" . compat_get_plugin_url( 'wordtwit' ) . "/css/banner.css\"></link>\n";
					echo "\t<script type=\"text/javascript\" src=\"" . get_bloginfo('wpurl') . "/wp-includes/js/jquery/jquery.js?ver=1.2.6\"></script>\n";
					echo "\t<script type=\"text/javascript\" src=\"" . compat_get_plugin_url( 'wordtwit' ) . "/js/banner.js\"></script>\n";
					echo "</head>\n";
					echo "<body>\n";

					echo "\t<div id=\"banner\"><img src=\"" . $profile['user']['profile_image_url'] . "\" alt=\"\" /><strong><a href=\"http://twitter.com/" . $profile['user']['screen_name'] . "\">Follow " . $profile['user']['name'] . " On Twitter</a></strong><p id=\"blog-url\"><a href=\"" . get_bloginfo('home') . "\">" . str_replace( 'www.', '', str_replace('http://', '', strtolower( get_bloginfo('home') ) ) ) . "</a></p>\n";
					echo "<p id=\"latest\">";
					echo "<h2>Latest posts from '" . get_bloginfo('name') . "'</h2>\n";
					echo "<ul>";
					$query = new WP_Query('showposts=3');
					$wti = 1;
					while ( $query->have_posts() ) { 
						$query->the_post();
						echo '<li class="banner-posts" id="banner-post-' . $wti . '"><a href="' . get_permalink() . '">' . get_the_title() . '</a></li>';
						$wti++;	
					}
					echo "</ul>\n";
					echo "</p>\n";
					echo "<div id=\"close-frame\"><img src=\"" . compat_get_plugin_url( 'wordtwit' ) . "/images/frame.png\" alt=\"\" /><a href=\"" . $result->original . "\">View Original</a></div>";						
					echo "</div>\n";
				
					echo "\t<div id=\"site-frame\"><iframe src=\"" . $result->original . "\" /></div>\n";					
					echo "</body>\n";
					echo "</html>\n";
					die;				
				} else {
					// internal site
					header( "HTTP/1.1 301 Moved Permanently" ); 
					header( "Location: " . $result->original ); 					
				}
				die;
			}	
		}
	}
}

function wordtwit_get_profile_url() {
	$settings = wordtwit_get_settings();
		
	return $settings['profile']['user']['profile_image_url'];
}

function wordtwit_get_username() {
	$settings = wordtwit_get_settings();
		
	return $settings['username'];	
}

function wordtwit_get_recent_tweets( $max_tweets = 5 ) {
   $snoopy = new Snoopy;
   $snoopy->agent = 'WordTwit ' . $wordtwit_version;

	$settings = wordtwit_get_settings();
   
   $last_fetch_time = get_option( 'wordtwit_last_fetch_time', 0 );
   $time_since_last = time() - $last_fetch_time;
   
   if ( $time_since_last > WORDTWIT_CACHE_TIME ) {	   
   	$xml_output = false;
   	
		$result = $snoopy->fetch( "http://search.twitter.com/search.atom?q=from:{$settings['username']}+-source:wordtwit" );
		if ( $result ) {
		   $output = $snoopy->results;  
		   
		   $xml_output = wordtwit_parsexml( $output );
		   
		   update_option( 'wordtwit_recent_tweets', $xml_output );
		}  
		
		update_option( 'wordtwit_last_fetch_time', time() );
		
		return @array_slice( $xml_output['feed']['entry'], 0, $max_tweets );
		
   } else {
   	$xml_output = get_option( 'wordtwit_recent_tweets', false );
   	return @array_slice( $xml_output['feed']['entry'], 0, $max_tweets );
   }
	
	return false;
}

function twit_hit_server( $location, $username, $password, &$output, $post = false, $post_fields = '' ) {
   global $wordtwit_version;
   $output = '';
   
   $snoopy = new Snoopy;
   $snoopy->agent = 'WordTwit ' . $wordtwit_version;
   
   if ( $username ) {
      $snoopy->user = $username;
      if ( $password ) {
         $snoopy->pass = $password;      
      }
   }
   
   if ( $post ) {
      // need to do the actual post
      $result = $snoopy->submit( $location, $post_fields );
      if ( $result ) {
			$output = $snoopy->results;
			return true;  
      }
   } else {
      $result = $snoopy->fetch( $location );
      if ( $result ) {
         $output = $snoopy->results;  
      }
      
      $code = explode( ' ', $snoopy->response_code );
      if ( $code[1] == 200) {
         return true;
      } else {
         return false;
      }
   }
}

function twit_get_ow_ly_url( $link ) {
        $output = $link;

        $result = twit_hit_server( 'http://ow.ly/url/shorten-url', '', '', $output, true, array( 'url' => $link, 'form_submit' => 'Shrink It' ) );

        if ( $result ) {
                if ( preg_match( '#<div class="ipt c-a"><input type="text" value="(.*)"#iUs', $output, $matches ) ) {
                	return $matches[1];
                } 
        }

        return $output;
}


function twit_update_status( $username, $password, $new_status ) {
   $output = '';
   return twit_hit_server( 'http://twitter.com/statuses/update.xml', $username, $password, $output, true, array( 'status' => $new_status, 'source' => 'wordtwit' ) );
}

function twit_verify_credentials( $username, $password, &$cred ) {
	global $twit_plugin_prefix;
   $output = '';
   
   $result = twit_hit_server( 'http://twitter.com/account/verify_credentials.xml', $username, $password, $output );  
   if ( $result ) {
        $cred = wordtwit_parsexml( $output );
        
        $settings = wordtwit_get_settings();
        $settings['profile'] = $cred;
        
        wordtwit_save_settings( $settings );
   } 
   
   return $result;
}

function twit_get_tiny_url( $link ) {
   $output = '';
   $result = twit_hit_server( 'http://tinyurl.com/api-create.php?url=' . $link, '', '', $output );
   
   return $output;
}

function twit_get_miudin_url( $link ) {
   $output = '';
   $result = twit_hit_server( 'http://miud.in/api-create.php?url=' . $link, '', '', $output );
   
   return $output;
}

function twit_get_bitly_url( $link ) {
	$settings = wordtwit_get_settings();
		
	$output = false;
	$result = twit_hit_server( 'http://api.bit.ly/shorten?version=2.0.1&longUrl=' . urlencode( $link ) . '&format=xml&history=1&login=' . $settings['bitly-user-name'] . '&apiKey=' . $settings['bitly-api-key'], '', '', $output );
	
   preg_match( '#<shortUrl>(.*)</shortUrl>#iUs', $output, $url );
	
	if ( isset( $url[1] ) ) {
		return $url[1];	
	} else {
		return false;
	}
}

function wordtwit_make_tinyurl( $link, $update = true, $post_id ) {
	if ( strpos( $link, 'http://' ) === false ) {
		return $link;	
	}
	
	$settings = wordtwit_get_settings();

	$short_link = $link;
	
	$utm_add_on = "utm_source=wordtwit&utm_medium=social&utm_campaign=wordtwit";
	if ( isset( $settings['enable-utm'] ) && $settings['enable-utm'] ) {
		if ( strpos( $link, "?" ) !== false ) {
			$link = $link . "&" . $utm_add_on;	
		} else {
			$link = $link . "?" . $utm_add_on;
		}
	}
	
	if ( $settings['url_type'] == 'tinyurl' ) { 
		$short_link =  twit_get_tiny_url( $link );			
	} else if ( $settings['url_type'] == 'miud.in' ) { 
		$short_link =  twit_get_miudin_url( $link );			
	} else if ( $settings['url_type'] == 'owly' ) {
		$short_link =  twit_get_ow_ly_url( $link );
	} else if ( $settings['url_type'] == 'local' ) {
		$short_link = wordtwit_tinyurl( $link, $update );				
	} else if ( $settings['url_type'] == 'bitly' ) {
		$short_link = twit_get_bitly_url( $link );
	} else if ( $settings['url_type'] == 'post_id' ) {
		if ( isset( $settings['enable-utm'] ) && $settings['enable-utm'] ) {
			$short_link = wordtwit_post_id_url_base() . $post_id . "&" . $utm_add_on; 		
		} else {
			$short_link = wordtwit_post_id_url_base() . $post_id; 				
		}
	}

	return $short_link;
}

function post_now_published( $post_id ) {
	global $twit_plugin_prefix;
	
	$settings = wordtwit_get_settings();
	
	$wt_tags = $settings['tags'];
	$wt_reverse = $settings['reverse'];
		
	$activation_time = $settings['activation_time'];
	$cur_time = time();

	query_posts( 'p=' . $post_id );
	if ( have_posts() ) {
		the_post();
		
		global $post;
		$cur_time = strtotime( $post->post_date );
		if ( $cur_time < $activation_time ) {
			//echo $cur_time . " " . $activation_time . " " . time() . " " ;
			//echo 'leaving'; die;
			// don't publish old posts
			return;
		}
		
		$can_tweet = true;
		
		// check tags
		if ( count( $wt_tags ) ) {
						
			// we have a tag or a category
			$new_taxonomy = array();
						
			$post_tags = get_the_tags();
			if ( $post_tags ) {
				foreach ( $post_tags as $some_tag ) {
					$new_taxonomy[] = strtolower( $some_tag->name );
				}
			}
			
			$post_categories = get_the_category();
			if ( $post_categories ) {
				foreach ( $post_categories as $some_category ) {
					$new_taxonomy[] = strtolower( $some_category->name );
				}
			}
			
			$category_hits = array_intersect( $wt_tags, $new_taxonomy );	
			
			if ( $wt_reverse ) {
				$can_tweet = ( count( $category_hits ) == 0);
			} else {
				$can_tweet = ( count( $category_hits ) > 0);
			}
		}

		$has_been_twittered = get_post_meta( $post_id, 'has_been_twittered', true );
		if ( $has_been_twittered !== 'yes' && $can_tweet ) {
			$i = 'New blog entry \'' . the_title('','',false) . '\' - ' . get_permalink();
			$message = $settings['message'];
			$message = str_replace( '[title]', $post->post_title, $message );
			
			global $post;
			$message = str_replace( '[link]', wordtwit_make_tinyurl( get_permalink(), true, $post->ID ), $message );			
			
			$twit_username = $settings['username'];
			$twit_password = $settings['password'];
			
			twit_update_status( $twit_username, $twit_password, $message );
	
			add_post_meta( $post_id, 'has_been_twittered', 'yes' );
		}
	}
}

function wordtwit_admin_css() {
	if ( isset( $_GET['page'] ) && $_GET['page'] == 'wordtwit.php' ) {
	echo '<link rel="stylesheet" type="text/css" href="' . compat_get_plugin_url('wordtwit') . '/css/wordtwit-admin.css" />';
	echo '<link rel="stylesheet" type="text/css" href="' . compat_get_plugin_url('wordtwit') . '/css/bnc-global.css" />';
	}
}

function bnc_stripslashes_deep( $value ) {
	$value = is_array($value) ?
   array_map('bnc_stripslashes_deep', $value) :
   stripslashes($value);
	return $value;
}

function wordtwit_options_subpanel() {
	if (get_magic_quotes_gpc()) {
    	 $_POST = array_map( 'bnc_stripslashes_deep', $_POST );
	    $_GET = array_map( 'bnc_stripslashes_deep', $_GET );
	    $_COOKIE = array_map( 'bnc_stripslashes_deep', $_COOKIE );
	    $_REQUEST = array_map( 'bnc_stripslashes_deep', $_REQUEST );
	}

	global $twit_plugin_name;
	global $twit_plugin_prefix;
	
	$settings = wordtwit_get_settings();

  	if ( isset($_POST['info_update']) ) {
		if ( isset($_POST['username'] ) ) {
			$username = $_POST['username'];
		} else {
			$username = '';
		}
		
		if ( isset( $_POST['enable_content_conversion'] ) ) {
			$settings['enable_content_conversion'] = true;	
		} else {
			$settings['enable_content_conversion'] = false;
		}

		if ( isset($_POST['password']) ) {
			$password = $_POST['password'];
		} else {
			$password = '';
		}

		if ( isset($_POST['message']) ) {
			$message = $_POST['message'];
		} else {
			$message = '';
		}
		
		if ( isset( $_POST['wordtwit-url-type'] ) ) {
			$settings['url_type'] = $_POST['wordtwit-url-type'];	
		}
		
		if ( isset( $_POST['enable_banner'] ) ) { 
			$settings['enable_banner'] = true;
		} else {
			$settings['enable_banner'] = false;	
		}		
		
		if ( isset( $_POST['bitly-api-key'] ) ) { 
			$settings['bitly-api-key'] = $_POST['bitly-api-key'];
		} else {
			$settings['bitly-api-key'] = '';	
		}		
		
		if ( isset( $_POST['bitly-user-name'] ) ) { 
			$settings['bitly-user-name'] = $_POST['bitly-user-name'];
		} else {
			$settings['bitly-user-name'] = '';	
		}			
		
		if ( isset( $_POST['alternate-domain'] ) ) {
			if ( strlen( $_POST['alternate-domain'] ) == 0 ) {
				$settings['alternate_domain'] = '';
			} else {		
				$settings['alternate_domain'] = $_POST['alternate-domain'];	
				$settings['alternate_domain'] = strtolower( rtrim( $settings['alternate_domain'], '/' ) );
				if ( strpos( $settings['alternate_domain'], 'http://' ) === false ) {
					$settings['alternate_domain'] = 'http://' . $settings['alternate_domain'];	
				}
			}
		}
		
		if ( isset( $_POST['wordtwit-reverse'] ) ) {
			$reverse = true;
		} else {
			$reverse = false;
		}
		
		if ( isset( $_POST['oauth_token'] ) ) {
			$settings['oauth_token'] = $_POST['oauth_token'];	
		} else {
			$settings['oauth_token'] = false;
		}
		
		if ( isset( $_POST['oauth_token_secret'] ) ) {
			$settings['oauth_token_secret'] = $_POST['oauth_token_secret'];	
		} else {
			$settings['oauth_token_secret'] = false;
		}		
		
		if ( isset( $_POST['oauth_pin'] ) ) {
			$settings['oauth_pin'] = $_POST['oauth_pin'];	
		} else {
			$settings['oauth_pin'] = false;
		}		
		
		if ( isset( $_POST['remove-www'] ) ) {
			$settings['remove-www'] = true;
		} else {
			$settings['remove-www'] = false;	
		}
			
		if ( isset( $_POST['enable-utm'] ) ) {
			$settings['enable-utm'] = true;
		} else {
			$settings['enable-utm'] = false;	
		}		
		
		$wt_tags = explode( ",", $_POST['wordtwit-tags'] );

		$settings['username'] = $username;
		$settings['password'] = $password;
		
		$new_tags = array();
		foreach ( $wt_tags as $tags ) {
			$clean_tag = strtolower( rtrim( ltrim( $tags ) ) );
			if ( strlen( $clean_tag ) ) {
				$new_tags[] = 	$clean_tag;
			}
		}
		
		$settings['tags'] = $new_tags;
		$settings['message'] = stripslashes( $message );
		$settings['reverse'] = $reverse;
		
		wordtwit_save_settings( $settings );
	} 
	
	// The Master Kill Switch
 	elseif ( isset( $_POST['reset'] ) ) {
		update_option( 'wordtwit_settings', '' );
    }

   include( 'html/options.php' );
}

function wordtwit_save_settings( $settings ) {
	update_option( 'wordtwit_settings', $settings );
}

function wordtwit_check_table() {
	global $table_prefix;
	global $wpdb;
	
	$table_name = $table_prefix . 'tweet_urls';
	$sql = "describe $table_name";
	$result = $wpdb->get_results( $sql );
	if ( !$result ) {
		// create table		
		$sql = "CREATE TABLE `" . $table_name . "` (" .
		  "`id` int(10) unsigned NOT NULL auto_increment," .
		  "`original` varchar(512) collate utf8_bin default NULL," .
		  "`url` varchar(7) collate utf8_bin NOT NULL," .
		  "`views` int(11) NOT NULL default '0'," .
		  "`post_id` int(11) NOT NULL default '0'," .
		  "PRIMARY KEY  (`id`)," .
		  "UNIQUE KEY `url_index` (`url`)," .
		  "KEY `original` (`original`)," .
		  "KEY `url_index_2` (`url`)" .
		") ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_bin;";
		
		$result = $wpdb->query( $sql );
	}
}

function wordtwit_post_id_url_base() {
	$settings = wordtwit_get_settings();
	$url = get_bloginfo( 'home' ) . '/?p=';
	if ( isset( $settings['remove-www'] ) && $settings['remove-www'] ) {
		$url = str_replace( 'www.', '', $url );
	}
	
	return $url;
}

function wordtwit_cache_tweets() {
	global $twit_plugin_prefix;
	global $table_prefix;
	global $wpdb;
	
	$table_name = $table_prefix . 'tweetcache';	
	
	wordtwit_check_table();
	
	$username = get_option( $twit_plugin_prefix . 'username' );
	$password = get_option( $twit_plugin_prefix . 'password' );	
	
	if ( $username && $password ) {
		$content = '';
		$result = twit_hit_server( 'http://twitter.com/statuses/replies.xml', $username, $password, $content );
		//$result = twit_hit_server( 'http://duanestorey.com/test.xml', $username, $password, $content );
		if ( $result ) {
			$xml = wordtwit_parsexml( $content );
			foreach( $xml['statuses']['status'] as $status ) {
				$sql = "select * from $table_name where from_status_id = '" . $status["id"] . "'";
				$result = $wpdb->get_row( $sql );
				if ( !$result ) {
					$from_name = $status["user"]["name"];
					$from_screen_name = $status["user"]["screen_name"];
					$profile_pic = $status["user"]["profile_image_url"];
					$from_date_time = strtotime( $status["created_at"] );
					$id = $status["id"];
					$reply_to = $status["in_reply_to_status_id"];
					$tweet = substr( $status["text"], strlen( $username ) + 2, strlen( $status["text"] ) - strlen( $username ) - 2);
					$home_url = $status["user"]["url"];

					if ( is_array( $status["user"]["url"] ) && count( $status["user"]["url"] ) == 0 ) {
							$home_url = '';					
					} else {
							$home_url = (string)$status["user"]["url"];
					}
					
					if ( is_array( $status["user"]["location"] ) && count( $status["user"]["location"] ) == 0 ) {
							$from_location = '';					
					} else {
							$from_location = (string)$status["user"]["location"];
					}					
					
					$sql = $wpdb->prepare( "INSERT INTO $table_name ( from_name, from_profile_pic, from_location, from_date_time, from_status_id, from_in_reply_to, from_tweet, from_home_url, from_screen_name, imported ) VALUES ( %s, %s, %s, %d, %d, %d, %s, %s, %s, %d )", $from_name, $profile_pic, $from_location, $from_date_time, $id, $reply_to, $tweet, $home_url, $from_screen_name, 0 );
					
					$result = $wpdb->query( $sql );
				} else {
					// We already have this in the database.
					// Probably safe to assume that we have everything else after it since the feed is hopefully ordered by date.
					break;
				}
			}
		}
				
		$result = twit_hit_server( 'http://twitter.com/statuses/user_timeline/' . $username . '.xml', $username, $password, $content );
		if ( $result ) {
			$table_name = $table_name . "_local";
			$xml = wordtwit_parsexml( $content );	
			$profile_url = '';
				
			foreach( $xml['statuses']['status'] as $status ) {
				$sql = "select * from $table_name where status_id = '" . $status["id"] . "'";		
				$result = $wpdb->get_row( $sql );
				if ( !$result ) {		
					$tweet = $status["text"];
					$date_time = strtotime( $status["created_at"] );	
					$id = $status["id"];		
					
					if ( !$profile_url ) {
						$profile_url = $status["user"]["profile_image_url"];	
					}				
						
					$sql = $wpdb->prepare( "INSERT INTO $table_name ( status_id, tweet, date_time ) VALUES ( %d, %s, %d )", $id, $tweet, $date_time );
						
					$result = $wpdb->query( $sql );		
				} else {
					break;	
				}										
			}		
			
			if ( $profile_url ) {
				update_option( $twit_plugin_prefix . 'profile_pic', $profile_url );
			}	
		}		
	} 
}

function wordtwit_add_plugin_option() {
	global $twit_plugin_name;
	if (function_exists('add_options_page')) {
		add_options_page($twit_plugin_name, $twit_plugin_name, 9, basename(__FILE__), 'wordtwit_options_subpanel');
   }	
}

function wordtwit_shortcode( $data ) {
	
	global $twit_plugin_prefix;
	global $table_prefix;
	global $wpdb;
	
	$table_name = $table_prefix . 'tweetcache';
	$profile_pic = get_option( $twit_plugin_prefix . 'profile_pic' );
	
	$username = get_option( $twit_plugin_prefix . 'username');
	
	$content = '';
		
	if ( isset( $data['status'] ) ) {
		$content .= "<h3>Thoughts From Twitter</h3><div class=\"wordtwit_tweets\"><ul>\n";

		$status = $data['status'];
		$sql = "select * from " . $table_name . "_local where status_id = " . $status;
		$result = $wpdb->get_row( $sql );
		if ( $result ) {
			$content .= "\t<li class=\"original\">\n";
			$content .= "\t\t<img src=\"" . $profile_pic . "\" alt=\"\" />\n";
			$content .= "\t\t<em><a href=\"http://twitter.com/" . $username . "\">" . $username . "</a> - " . date( "F jS, Y @ g:ia", $result->date_time ) . "</em>\n";
			$content .= "\t\t" . $result->tweet . "\n";
			$content .= "\t\t<div class=\"join\"><a href=\"http://twitter.com/home?status=@" . $username . "%20&in_reply_to_status_id=" . $status . "&in_reply_to=" . $username . "\">Join The Discussion</a> | <a href=\"#\" onclick=\"jQuery('.response').fadeOut(); return false;\">Hide</a></div>\n";
			$content .= "\t\t<div class=\"wordtwit_clearer\"></div>\n";
			$content .= "\t</li>\n";				
		}
		
		$sql = "select * from " . $table_name . " where from_in_reply_to = " . $status . " and imported = 0 order by from_date_time asc";
		$results = $wpdb->get_results( $sql );
		
		global $post;
		if ( $results ) {
			foreach( $results as $result ) {
				$content .= "\t<li class=\"response\">\n";
				$content .= "<img src=\"" . $result->from_profile_pic . "\" alt=\"\" />\n";
				$content .= "\t\t<em><a href=\"http://twitter.com/" . $result->from_screen_name . "\" rel=\"nofollow\">" . $result->from_name . "</a> - " . date( "F jS, Y @ g:ia", $result->from_date_time ) . "</em>\n";
				$content .= "\t\t" . $result->from_tweet . "\n";
				$content .= "\t\t<div class=\"wordtwit_clearer\"></div>\n";
				$content .= "\t</li>\n";	
				
				$sql = "select * from " . $table_prefix . "comments where comment_karma = " . $result->from_status_id;
				$comment = $wpdb->get_row( $sql );
				if ( !$comment ) {
					$comment_data = array();
					$comment_data['comment_karma'] = $result->from_status_id;
					$comment_data['comment_post_ID'] = $post->ID;
					$comment_data['comment_author'] = $result->from_name;
					$comment_data['comment_author_url'] = "http://twitter.com/" . $result->from_screen_name;
					$comment_data['comment_author_email'] = $result->from_screen_name . "@wordtwit.com";
					$comment_data['comment_content'] = $result->from_tweet;
					$comment_data['comment_type'] = 'wordtwit';
					
					$comment_id = wp_new_comment( $comment_data ); 
					if ( $comment_id ) {
						$sql = "update " . $table_prefix . "comments set comment_karma = " . $result->from_status_id . ", comment_agent = '" . $result->from_profile_pic . "' where comment_ID = " . $comment_id;
						$wpdb->query( $sql );
						
						$sql = $wpdb->prepare( "UPDATE " . $table_prefix . "tweetcache SET imported = 1 WHERE from_status_id = %d", $result->from_status_id );
						$wpdb->query( $sql );
					}
				}
			}
		}
		
		$content .= "</ul></div>\n";
		
		if ( is_single() ) print_r( $comments );
	}
	
	//return $content;
	return '';
}	

function wordtwit_content( $content ) {
	preg_match_all( "'href=\"?(.[^\"\>]*)\"?(.[^\>]*)>(.[^\<]*)</a>'im",$content,$matches );
	if ( $matches ) {
		foreach ( $matches[1] as $url ) {
			$update_post_num = false;
			if ( strtolower( $url ) == strtolower( get_permalink() ) ) {
				$update_post_num = true;
			}
			
			$tinyurl = wordtwit_make_tinyurl( $url, $update_post_num );
			$content = str_replace( $url, $tinyurl, $content );
		}	
	}
	
	return $content;
}
add_action( 'admin_menu', 'wordtwit_add_plugin_option' );
add_filter( 'plugin_action_links', 'wordtwit_settings_link', 9, 2 );
//add_shortcode( 'wordtwit', 'wordtwit_shortcode');
?>
